###################################################################################
#####Code to run the backward iteration (SDP) of the model for ENP blue whale######
###################################################################################

#Load required libraries----
library(Rcpp)
library(RcppArmadillo)
library(RcppProgress)

#Load SDP functions (Rcpp)----
#separate functions are available for each reproductive state (1-4), and specific functions are used for environmental (te) and state (m/maturation, i/implantation, b/birth, w/weaning) transitions
sourceCpp(file='Rcpp functions/BW_Rcpp_fun1.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun1m.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun1te.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun2.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun2i.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun2te.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun3.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun3b.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun3te.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun4.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun4w.cpp')
sourceCpp(file='Rcpp functions/BW_Rcpp_fun4te.cpp')

#Set locations----
locs<-36      #number of locations
feed_g<-15    #this is the location at which the main summer feeding grounds start (California current)
southloc<-10  #this is the location below which females are constrained to stay for birth and early stages of lactation
d<-100        #distance (km) females can cover in a day of travelling

#Set important ages---- 
aM<-6      #age at sexual maturity
N<-90      #number of reproductive years before death or senescence 
maxA<-aM+N #maximum age

#Set important times of the year----
tb<-1                #females can be born/give birth on 1 Dec 
ti<-32               #they can implant on 1 Jan
tw<-212              #at 7 months, a calf can start feeding autonomously
te1<-31              #day on which the environment can switch (31 Dec)
ds<-40               #day after birth in which the mother and the calf can leave the breeding ground
ns<-0:N              #years from maturity to death
ns2<-0:(aM+N)        #years from birth
tbs<-tb+(aM +ns)*365 #model days on which calves can be born
tis<-ti+(aM +ns)*365 #model days on which a female can implant
tws<-tw+(aM +ns)*365 #model days on which calves can get weaned
tes1<-te1+ns2*365    #model days on which the environment can switch can occur

#General model settings----
T<-tw+(aM+N)*365                             #time horizon
nstates<-4                                   #number of reproductive states (1-Juvenile, 2-Resting adult, 3-Pregnant, 4-Lactating)
t_il<-(tws-1)-length((T-1):tbs[length(tbs)]) #used to extract the day in lactation (1-212)

#Survival/mortality----
surv<-0.975^(1/365)      #daily probability of survival
calf_s<-0.875^(1/212)    #daily probability of calf survival
fet_s<-(1-0.135)^(1/334) #daily probability of fetus survival

#Pregnancy----
gtime<-1:334                      #gestation is 11 months
Wc<-((0.52*(gtime-73))^3)/1000    #foetal growth for blue whales in kg
Wc[Wc<0.001]<-0                   #set weight of foetus to 0 when less than 1g
cost_kg<-2940*4.184               #kJ/kg of foetus weight
Wc_d<-(c(Wc[-1],NA))-Wc           #increase in foetus weight (kg) each day
propg<-Wc_d/max(Wc)               #proportional growth on each day
Chg_tot<-4400*4.184*(max(Wc)^1.2) #total cost of heat of gestation
Chg<-propg*Chg_tot                #heat of gestation (in kJ/day) depending on day of pregnancy (i.e on proportional growth on that day)

#Lactation and calf growth----
Mmax<-220                            #maximum milk delivered daily (kg)
me<-4137*4.184                       #energy content (kJ/kg) of milk
E<-0.9                               #90% mammary gland efficiency 
wb<-2500                             #weight at birth (kg) (from gestation growth curve)
lb<-2500*0.94                        #lean mass at birth (assuming calf is mostly lean at birth), but blubber % needs to be one interval above threshold for death
ww<-17000+2500                       #average estimated growth + mass at birth (i.e. mass at weaning)
lw<-ww*0.73                          #lean mass at weaning 
lgd<-(lw-lb)/211                     #calf average daily growth in lean mass; kg/d
leand<-c(lb,lb+cumsum(rep(lgd,211))) #total lean mass on each day, including the first day
meat_cal<-1500*4.184                 #calorific content of whale lean tissue (kJ/kg)
LEd<-lgd*meat_cal                    #kJ required to grow each day
MRc<-200000*4.184                    #average kJ to sustain maintainance/metabolic rate
Er<-MRc + LEd                        #total daily amount of energy required by the calf to survive and grow

#Feeding and resting----
u<-31798                   #utilizable energy (kJ/kg) in blubber 
w<-c(10,20,30)             #number of lunges per hour (depending on krill density: <0.2, 0.2-0.4, >0.4)
tc<-4                      #number of hours to clear forestomach
rho<-4184                  #amount of energy in krill (kJ/kg)
A<-0.84                    #assimilation efficiency 
for_hours<-read.csv("Ancillary files/Twilight_times_byLoc.csv",header=T) #total number of hours available for foraging during a day (not feeding at night)
for_hours<-rbind(for_hours[1,],for_hours)                #same hours in Gulf of California as off the southern tip of Baja
for_hoursn<-t(for_hours)
for_hoursn2<-c(for_hoursn)                               #vector for C++

#Growth after weaning----
ts<-(tw+1):T                          #time in days starting on day after weaning 
lt<-22*(1-exp(-0.240*((ts/365)+4.5))) #females grow asymptotically towards average length at maturity (22 m)
leant<-0.73*4.6*(lt)^3.05             #lean mass on each day
diff_l<-(leant[3]-leand[length(leand)])/3     #smooth transition of lean mass from calf to juvenile
leant[1:3]<-leand[length(leand)]+diff_l*(1:3) #smooth transition of lean mass from calf to juvenile
lean_complete<-c(leand,leant)                 #complete time series of lean mass (calf, juvenile and adult)
leant_day<-round(lean_complete[2:length(lean_complete)]-lean_complete[1:(length(lean_complete)-1)]) #amount of lean mass accumulated on each day
per_lean<-seq(0.94,0.73,length.out=length(leand)) #the ratio lean/blubber at birth is NOT 0.73, but 0.94. This ratio is likely to progressively approach the adult ratio as lactation progresses
lt_prewean<-(leand/per_lean/4.6)^(1/3.05)         #length pre-weaning
lt_complete<-c(lt_prewean,lt)                     #complete time series of lenght

#Extremes for m state variable (calf condition, i.e. the proportion of calf mass that is blubber)----
mc<-0.05                     #min % of blubber mass (at 5% blubber mass, calf dies)
mC<-0.44                     #max % of blubber mass (calf cannot have more than 44% fat)
int_m<-0.03                  #calf condition bins
ms<-seq(mc,mC+int_m,by=int_m)#vector of possible calf conditions
m_tok<-10                    #token value of m state variable for all reproductive states other than 4

#Extremes for x state variable (blubber mass) - with adaptive bin size----
int_x<-2000                               #bin size for large blubber masses (coarser)
int_low_x<-500                            #minimum bin size (when smaller mass; finer)
max_perc<-rep(0.35,length(lean_complete)) #time series of maximum observed % of blubber in the body
max_perc[1:tw]<-0.44                      #maximum during lactation period
max_perc[(tw+1):tbs[1]]<-seq(0.44,0.35,length.out=length((tw+1):tbs[1]))  #in the first 2 years, the maximum storage abilities linearly decline from calf's (0.44) to adult's (0.35)
min_perc<-0.16                            #minimum observed % of blubber in the body (used as threshold for lactation)
death_perc<-0.05                          #% of blubber at which the whale dies
min_xc<-round(death_perc*lean_complete[1]/(1-death_perc)/int_low_x)*int_low_x                                                      #min value of blubber mass across life  
max_C<-round(max_perc[length(lean_complete)]*lean_complete[length(lean_complete)]/(1-max_perc[length(lean_complete)])/int_x)*int_x #max value of blubber mass across life
xs_wide<-round(unique(c(seq(min_xc,min_xc+int_x,length.out=5), #vector of possible female conditions
           seq(min_xc+int_x,min_xc+int_x*2,length.out=5),
           seq(min_xc+int_x*2,min_xc+int_x*3,length.out=4),
           seq(min_xc+int_x*3,min_xc+int_x*4,length.out=3),
           seq(min_xc+int_x*4,min_xc+int_x*5,length.out=4),
           seq(min_xc+int_x*5,min_xc+int_x*6,length.out=3),
           seq(min_xc+int_x*5,max_C+int_x,by=int_x))),-2) 
Ctmp<-max_perc*lean_complete/(1-max_perc)       
x_lactmp<-min_perc*lean_complete/(1-min_perc)   
xctmp<-death_perc*lean_complete/(1-death_perc)  
C<-xs_wide[sapply(Ctmp,function(x) which.min(abs(x-xs_wide)))]   #time series of maximum blubber masses
xc<-xs_wide[sapply(xctmp,function(x) which.min(abs(x-xs_wide)))] #time series of minimum bkubber masses
x_lac<-xs_wide[sapply(x_lactmp,function(x) which.min(abs(x-xs_wide)))]

#Krill densities----
mdens<-rgamma(1000000,2.5,5.5)*0.8              #distribution of krill densities in California, in summer, for a swarm of E. pacifica + T. spinifera (0.8 is a scaling factor to adjust overall environmental productivity)
mdens<-mdens[mdens<quantile(mdens,probs=0.999)] #exclude very extreme values of the distribution
lag<-90                                         #time lag between upwelling and krill peaks 
gcal_lag<-121+lag-74                            #lag for Gulf of California
upw<-read.csv("Ancillary files/Upwelling_Index.csv",header=T)   #time series of upwelling index (NOAA)
gcal<-data.frame(julday=0:365,loc=0,pr=0,se=0,pr_scaled=c(upw$pr_scaled[upw$loc==unique(upw$loc)[1]][gcal_lag:366],upw$pr_scaled[upw$loc==unique(upw$loc)[1]][1:(gcal_lag-1)]))
upw<-rbind(gcal,upw)                            #add upwelling for Gulf of California
upw$pr_scaled[upw$pr_scaled<0]<-0               
for (i in unique(upw$loc)[c(1,6:11)]){
 upw[upw$loc==i,]<-rbind(upw[upw$loc==i,][(366-lag+1):366,],upw[upw$loc==i,][1:(366-lag),])
 }
coords<-read.csv("Ancillary files/Locations_coordinates.csv",header=T) #coordinates of modelled locations
coords<-rbind(c(0,0),coords)                           #add extra line for Gulf of California
yj<-array(0,dim=c(2,3,30,locs+1,366))                  #environmental productivity - 2 environmental regimes, 3 patch types, 30 values to capture distribution, 36 locations (one extra only for interpolating purposes), 365 days (one extra for interpolating purposes)
pj<-array(1/30,dim=c(2,3,30,locs+1,366))               #associated probabilities (from gamma distribution)
rsize<-0.6                                             #size ratio of krill species
#Baseline environmental conditions 
breaks<-cut(mdens*rsize,30)
yj[1,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
pj[1,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
breaks<-cut(mdens,30)
yj[1,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
pj[1,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
patch<-1
for (i in 1:locs){
 if(i>=feed_g){
  patch<-2
  }
 cl<-unique(upw$loc)[which.min(abs(coords[i,2]-unique(upw$loc)))]
 scaling<-upw$pr_scaled[upw$loc==cl]
 yj[1,patch,1:30,i,]<-t(t(yj[1,patch,1:30,i,])*scaling)
 }
#Unfavourable environmental conditions 
breaks<-cut(mdens*rsize,30)
yj[2,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
pj[2,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
breaks<-cut(mdens,30)
yj[2,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
pj[2,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
for (i2 in 1:locs){
 if(i2>=feed_g & i2<=28){scal_Nino<-0.3}else{scal_Nino<-1}  #in southern and central california current, productivity is less (extra scaling - reduction in production by 70%)
 cl<-unique(upw$loc)[which.min(abs(coords[i2,2]-unique(upw$loc)))]
 scaling<-upw$pr_scaled[upw$loc==cl]
 yj[2,1,1:30,i2,]<-t(t(yj[2,1,1:30,i2,])*scaling*scal_Nino) 
 yj[2,2,1:30,i2,]<-t(t(yj[2,2,1:30,i2,])*scaling*scal_Nino) 
 }
pNo<-0.2     #probability of occurrence of an El Nino year over the lifetime of a female
yjn<-aperm(yj,c(2,1,3,4,5))
yjn2<-c(yjn) #vector for C++
pjn<-aperm(pj,c(2,1,3,4,5))
pjn2<-c(pjn) #vector for C++

#Probabilities of encountering patches of different type when travelling in different macroareas---- 
#Baseline environmental conditions
lambda<-array(0,dim=c(2,locs+1,3,365))
lambda[1,1:(feed_g-1),1,]<-0.6
lambda[1,1:(feed_g-1),3,]<-0.4
lambda[1,feed_g:locs,2,]<-0.7
lambda[1,feed_g:locs,3,]<-0.3
lambda[1,locs+1,3,]<-1        #outside modelled range, only used for interpolation
#Unfavourable environmental conditions
maxNS<-28 #max location at which N. simplex is available
red2<-0.3 #reduction in availability of T.spinifera and E.pacifica in affected range
lambdatmp1<-0.6
lambdatmp2<-0.7
lambda[2,1:(feed_g-1),1,]<-lambdatmp1
lambda[2,1:(feed_g-1),3,]<-1-lambdatmp1
lambda[2,feed_g:maxNS,2,]<-lambdatmp2*red2                  #in region where range of N.simplex shifts
lambda[2,feed_g:maxNS,3,]<-1-lambdatmp2                     #in region where range of N.simplex shifts
lambda[2,feed_g:maxNS,1,]<-1-(1-lambdatmp2)-lambdatmp2*red2 #in region where range of N.simplex shifts
lambda[2,(maxNS+1):locs,2,]<-lambdatmp2
lambda[2,(maxNS+1):locs,3,]<-1-lambdatmp2
lambda[2,locs+1,3,]<-1        #outside modelled range, only used for interpolation
lambdan<-aperm(lambda,c(2,1,3,4))
lambdan2<-c(lambdan)          #vector for C++
rm(lambda, lambdan, yj, yjn, pj, pjn, for_hours, for_hoursn)
gc()

#Terminal fitness function (at t=T)---- 
gamma<-8  #steepness of fitness function
m50<-0.27 #% blubber mass of calf at which survival probability is 0.5
F1<-array(0,dim=c(2,length(xs_wide),length(ms),locs+1,3,nstates)) #initialise F1 for SDP (2 environmental regimes, x state variable, m state variable, loc state variable, patch state variable, reproductive states)
F1[,,dim(F1)[3],,,]<-(-99)                                        #set to -99 all the values that are not possible at time T (extra bins included only for interpolation purposes or bins that correspond to values of state variables that are not possible given age)
for (m in 2:(length(ms)-1)){
 F1[,,m,,,4]<-(ms[m]^gamma)/((ms[m]^gamma) + (m50^gamma))         #terminal fitness when weaning calf on last day of life
 }
F1[,,,dim(F1)[4],,]<-(-99)
F1[,1:max(which(xs_wide<xc[T])),,,,]<-(-99)         
F1[,min(which(xs_wide>C[T])):dim(F1)[2],,,,]<-(-99) 
F1n<-aperm(F1,c(2,1,3,4,5,6))
F1n2<-c(F1n) #vector for C++
F1tmp<-F1n2  #temporary F1, to be updated progressively as F0 changes
rm(F1,F1n)

#SDP backward iteration----

for (a in maxA:0){  #cycle backward over age in years

 #create arrays to store results (D/movement strategy, R/reproductive strategy, D2/second best movement strategy and F0/fitness values) for year a
 #array size: 2 environmental regimes, x state variable, m state variable, loc state variable, patch state variable, 4 states and 365 days
 resDl_n<-array(NA,dim=c(2,length(xs_wide),length(ms),locs,3,4,365))
 resDl_nn<-aperm(resDl_n,c(2,1,3,4,5,6,7))
 resDl_nn2<-c(resDl_nn)   #vector for C++
 resRl_n<-array(NA,dim=c(2,length(xs_wide),length(ms),locs,3,4,365))
 resRl_nn<-aperm(resRl_n,c(2,1,3,4,5,6,7))
 resRl_nn2<-c(resRl_nn)   #vector for C++
 rm(resDl_n,resDl_nn,resRl_n,resRl_nn)
 gc()
 resD2l_n<-array(NA,dim=c(2,length(xs_wide),length(ms),locs,3,4,365))
 resD2l_nn<-aperm(resD2l_n,c(2,1,3,4,5,6,7))
 resD2l_nn2<-c(resD2l_nn) #vector for C++
 resF0l_n<-array(NA,dim=c(2,length(xs_wide),length(ms),locs,3,4,365))
 resF0l_nn<-aperm(resF0l_n,c(2,1,3,4,5,6,7))
 resF0l_nn2<-c(resF0l_nn) #vector for C++
 rm(resD2l_n,resD2l_nn,resF0l_n,resF0l_nn)
 gc()
 #storage arrays are updated at each time step for each state (if applicable); F1 is updated at the end of each time step for all states
 
 #maximum and minimum days for the given year
 tmax<-ifelse(a!=maxA,365,tw-1) #maximum day for last year of life is different (female dies after last weaning)
 tmin<-ifelse(a!=0,1,tw)        #at the earliest, a new individual enters the population when capable of autonomous feeding (when weaned early, she needs to reach weaning time before she can feed autonomously and make behavioural decision on her own)

 for (dd in tmax:tmin){         #dd is the index indicating the day in the yearly cycle 
  t<-dd+a*365                   #model day (tw:T)

  #before sexual maturation
  if (t<(tbs[1]-1)){  
   if (!t%in%tes1){    #if NOT on days when environment can switch
    state<-1           #whale is a juvenile
    tmp<-fun_state1(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
    F1tmp<-tmp[[1]]
    resDl_nn2<-tmp[[2]]
    resRl_nn2<-tmp[[3]]
    resD2l_nn2<-tmp[[4]]
    resF0l_nn2<-tmp[[5]]
    }else{
    state<-1           #whale is a juvenile
    tmp<-fun_state1te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
    F1tmp<-tmp[[1]]
    resDl_nn2<-tmp[[2]]
    resRl_nn2<-tmp[[3]]
    resD2l_nn2<-tmp[[4]]
    resF0l_nn2<-tmp[[5]]
    }
   }

  #at sexual maturation (transition from juvenile to adult)
  if (t==(tbs[1]-1)){ 
   state<-1            #whale is a juvenile (at the start)
   tmp<-fun_state1mature(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
   F1tmp<-tmp[[1]]
   resDl_nn2<-tmp[[2]]
   resRl_nn2<-tmp[[3]]
   resD2l_nn2<-tmp[[4]]
   resF0l_nn2<-tmp[[5]]
   }

  #after sexual maturation 
  if (t>=tbs[1] & t<=(T-1)){
   for (state in 2:4){ #whale is an adult (3 possible states))     

     #on days between birth and implantation, or on the last year (after last birth): she can only be either resting or lactating 
     if ((dd>=tb & dd<ti) | (t>=tbs[length(tbs)])){
      if(t%in%tes1){   #if on days when environment can switch
       if (state==2){
        tmp<-fun_state2te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if(state==4){
        tmp<-fun_state4te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       }else{          #on days when environment cannot switch
       if(state==2){ 
        tmp<-fun_state2(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if(state==4 & t>tis[1]){ #she can't be lactating on the first year
        tmp<-fun_state4(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       }
      }

     #on implantation day (except last year, when she cannot implant)
     if (dd==ti & t!=tis[length(tis)]){
      if(state==2){ 
       tmp<-fun_state2implant(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
       F1tmp<-tmp[[1]]
       resDl_nn2<-tmp[[2]]
       resRl_nn2<-tmp[[3]]
       resD2l_nn2<-tmp[[4]]
       resF0l_nn2<-tmp[[5]]
       }
      if(state==4 & t!=tis[1]){ #on implantation days she can also be lactating, but not on the first year
       tmp<-fun_state4(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
       F1tmp<-tmp[[1]]
       resDl_nn2<-tmp[[2]]
       resRl_nn2<-tmp[[3]]
       resD2l_nn2<-tmp[[4]]
       resF0l_nn2<-tmp[[5]]
       }
      }

     #on days between implantation and next birth (she can be in any state), in all years except last
     if (dd>ti & dd<365 & t<tbs[length(tbs)]){
      if(t%in%tes1){  #if on days when environment can switch 
       if (state==2){
        tmp<-fun_state2te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if (state==3){
        tmp<-fun_state3te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if(state==4){
        tmp<-fun_state4te(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       }else{ 
       if(state==2){ 
        tmp<-fun_state2(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if(state==3){ 
        tmp<-fun_state3(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       if(state==4 & t>tws[1]){ 
        tmp<-fun_state4(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
        F1tmp<-tmp[[1]]
        resDl_nn2<-tmp[[2]]
        resRl_nn2<-tmp[[3]]
        resD2l_nn2<-tmp[[4]]
        resF0l_nn2<-tmp[[5]]
        }
       }
      }

     #on day before birth
     if (t %in% (tbs-1)){
      if (state==3){   #if she is pregnant, she can give birth (in the following time step)
       tmp<-fun_state3birth(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
       F1tmp<-tmp[[1]]
       resDl_nn2<-tmp[[2]]
       resRl_nn2<-tmp[[3]]
       resD2l_nn2<-tmp[[4]]
       resF0l_nn2<-tmp[[5]]
       }
      if(state==2){ 
       tmp<-fun_state2(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
       F1tmp<-tmp[[1]]
       resDl_nn2<-tmp[[2]]
       resRl_nn2<-tmp[[3]]
       resD2l_nn2<-tmp[[4]]
       resF0l_nn2<-tmp[[5]]
       }
      if(state==4 & t>tbs[2]){ #on birth days she can also be already lactating, but not on the first (just matured) and second (she cannot be lactating because hasn't given birth to any calf yet) year. She weans calf on that day if she hasn't before
       tmp<-fun_state4wean(yjn2,pjn2,for_hoursn2,lambdan2,a,dd,F1tmp,F1n2,resDl_nn2,resRl_nn2,resD2l_nn2,resF0l_nn2) #apply corresponding function to extract optimal behavioural and reproductive decisions
       F1tmp<-tmp[[1]]
       resDl_nn2<-tmp[[2]]
       resRl_nn2<-tmp[[3]]
       resD2l_nn2<-tmp[[4]]
       resF0l_nn2<-tmp[[5]]
       }
      }

     }#close state loop
    }#close post-maturation loop
  
  F1n2<-F1tmp #update F1
  print(t)    #keep track of SDP progress
  }#close day loop

 #save yearly arrays
 namD<-paste("Backward SDP results/ResultsSDP_strategyD_year",a,".RData",sep="")
 namR<-paste("Backward SDP results/ResultsSDP_strategyR_year",a,".RData",sep="")
 namD2<-paste("Backward SDP results/ResultsSDP_strategyD2_year",a,".RData",sep="")
 namF<-paste("Backward SDP results/ResultsSDP_fitnessF_year",a,".RData",sep="")
 save(resDl_nn2,file=namD)
 save(resRl_nn2,file=namR)
 save(resD2l_nn2,file=namD2)
 save(resF0l_nn2,file=namF)
 
 }#close year loop


